"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var server_1 = __importDefault(require("react-dom/server"));
var stream_1 = require("stream");
var ComponentRegistry_1 = __importDefault(require("./ComponentRegistry"));
var createReactOutput_1 = __importDefault(require("./createReactOutput"));
var isServerRenderResult_1 = require("./isServerRenderResult");
var buildConsoleReplay_1 = __importDefault(require("./buildConsoleReplay"));
var handleError_1 = __importDefault(require("./handleError"));
var serverRenderUtils_1 = require("./serverRenderUtils");
var stringToStream = function (str) {
    var stream = new stream_1.PassThrough();
    stream.write(str);
    stream.end();
    return stream;
};
var transformRenderStreamChunksToResultObject = function (renderState) {
    var consoleHistory = console.history;
    var previouslyReplayedConsoleMessages = 0;
    var transformStream = new stream_1.PassThrough({
        transform: function (chunk, _, callback) {
            var htmlChunk = chunk.toString();
            var consoleReplayScript = (0, buildConsoleReplay_1.default)(consoleHistory, previouslyReplayedConsoleMessages);
            previouslyReplayedConsoleMessages = (consoleHistory === null || consoleHistory === void 0 ? void 0 : consoleHistory.length) || 0;
            var jsonChunk = JSON.stringify((0, serverRenderUtils_1.createResultObject)(htmlChunk, consoleReplayScript, renderState));
            this.push("".concat(jsonChunk, "\n"));
            callback();
        }
    });
    var pipedStream = null;
    var pipeToTransform = function (pipeableStream) {
        pipeableStream.pipe(transformStream);
        pipedStream = pipeableStream;
    };
    // We need to wrap the transformStream in a Readable stream to properly handle errors:
    // 1. If we returned transformStream directly, we couldn't emit errors into it externally
    // 2. If an error is emitted into the transformStream, it would cause the render to fail
    // 3. By wrapping in Readable.from(), we can explicitly emit errors into the readableStream without affecting the transformStream
    // Note: Readable.from can merge multiple chunks into a single chunk, so we need to ensure that we can separate them later
    var readableStream = stream_1.Readable.from(transformStream);
    var writeChunk = function (chunk) { return transformStream.write(chunk); };
    var emitError = function (error) { return readableStream.emit('error', error); };
    var endStream = function () {
        transformStream.end();
        pipedStream === null || pipedStream === void 0 ? void 0 : pipedStream.abort();
    };
    return { readableStream: readableStream, pipeToTransform: pipeToTransform, writeChunk: writeChunk, emitError: emitError, endStream: endStream };
};
var streamRenderReactComponent = function (reactRenderingResult, options) {
    var componentName = options.name, throwJsErrors = options.throwJsErrors;
    var renderState = {
        result: null,
        hasErrors: false,
        isShellReady: false
    };
    var _a = transformRenderStreamChunksToResultObject(renderState), readableStream = _a.readableStream, pipeToTransform = _a.pipeToTransform, writeChunk = _a.writeChunk, emitError = _a.emitError, endStream = _a.endStream;
    var renderingStream = server_1.default.renderToPipeableStream(reactRenderingResult, {
        onShellError: function (e) {
            var error = (0, serverRenderUtils_1.convertToError)(e);
            renderState.hasErrors = true;
            renderState.error = error;
            if (throwJsErrors) {
                emitError(error);
            }
            var errorHtml = (0, handleError_1.default)({ e: error, name: componentName, serverSide: true });
            writeChunk(errorHtml);
            endStream();
        },
        onShellReady: function () {
            renderState.isShellReady = true;
            pipeToTransform(renderingStream);
        },
        onError: function (e) {
            if (!renderState.isShellReady) {
                return;
            }
            var error = (0, serverRenderUtils_1.convertToError)(e);
            if (throwJsErrors) {
                emitError(error);
            }
            renderState.hasErrors = true;
            renderState.error = error;
        },
    });
    return readableStream;
};
var streamServerRenderedReactComponent = function (options) {
    var componentName = options.name, domNodeId = options.domNodeId, trace = options.trace, props = options.props, railsContext = options.railsContext, throwJsErrors = options.throwJsErrors;
    try {
        var componentObj = ComponentRegistry_1.default.get(componentName);
        (0, serverRenderUtils_1.validateComponent)(componentObj, componentName);
        var reactRenderingResult = (0, createReactOutput_1.default)({
            componentObj: componentObj,
            domNodeId: domNodeId,
            trace: trace,
            props: props,
            railsContext: railsContext,
        });
        if ((0, isServerRenderResult_1.isServerRenderHash)(reactRenderingResult) || (0, isServerRenderResult_1.isPromise)(reactRenderingResult)) {
            throw new Error('Server rendering of streams is not supported for server render hashes or promises.');
        }
        return streamRenderReactComponent(reactRenderingResult, options);
    }
    catch (e) {
        if (throwJsErrors) {
            throw e;
        }
        var error = (0, serverRenderUtils_1.convertToError)(e);
        var htmlResult = (0, handleError_1.default)({ e: error, name: componentName, serverSide: true });
        var jsonResult = JSON.stringify((0, serverRenderUtils_1.createResultObject)(htmlResult, (0, buildConsoleReplay_1.default)(), { hasErrors: true, error: error, result: null }));
        return stringToStream(jsonResult);
    }
};
exports.default = streamServerRenderedReactComponent;
